//-	---------------------------------------------------------------------------
//-	Name: 		SDIGridPropPage.cpp
//-
//-	Purpose:	This file provides the implementation of the class
//-				CSDIGridPropPage.
//-
//-	Revisions:
//-	Date		Author		Change Summary
//-	---------	------		---------------------------------------------------
//-	09-30-99	Mklombardi@ Created & Released.
//-             netzero,net 
//-
//-	Notes:		None.
//-
//- Copyright : This software is NOT copyrighted.
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then acknowledgement must be made to the author of this file 
// (in whatever form you wish).
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy. Please let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into this
// file. 
//
//-	---------------------------------------------------------------------------
// SDIGridPpg.cpp : Implementation of the CSDIGridPropPage property page class.

#include "stdafx.h"
#include "SDIGrid.h"
#include "SDIGridPpg.h"
#include "SDIGridCtl.h"
#include "SetColumn.h"
#include "SetRowHeading.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CSDIGridPropPage, COlePropertyPage)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CSDIGridPropPage, COlePropertyPage)
	//{{AFX_MSG_MAP(CSDIGridPropPage)
	ON_BN_CLICKED(IDC_SET_COLUMN_BUTTON, OnSetColumnButton)
	ON_BN_CLICKED(IDC_SET_ROW_BUTTON, OnSetRowButton)
	ON_EN_CHANGE(IDC_NUM_COLS, OnChangeNumCols)
	ON_EN_CHANGE(IDC_NUM_ROWS, OnChangeNumRows)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CSDIGridPropPage, "SDIGRID.SDIGridPropPage.1",
	0xc3e95cf7, 0x64bc, 0x11d3, 0x8d, 0x9c, 0x4e, 0xe4, 0xb9, 0, 0, 0)


/////////////////////////////////////////////////////////////////////////////
// CSDIGridPropPage::CSDIGridPropPageFactory::UpdateRegistry -
// Adds or removes system registry entries for CSDIGridPropPage

BOOL CSDIGridPropPage::CSDIGridPropPageFactory::UpdateRegistry(BOOL bRegister)
{
	if (bRegister)
		return AfxOleRegisterPropertyPageClass(AfxGetInstanceHandle(),
			m_clsid, IDS_SDIGRID_PPG);
	else
		return AfxOleUnregisterClass(m_clsid, NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CSDIGridPropPage::CSDIGridPropPage - Constructor

CSDIGridPropPage::CSDIGridPropPage() :
	COlePropertyPage(IDD, IDS_SDIGRID_PPG_CAPTION)
{
	//{{AFX_DATA_INIT(CSDIGridPropPage)
	m_numRows = 0;
	m_numCols = 0;
	m_ColZeroIsFixed = FALSE;
	m_Address = 0;
	m_FixedColHeight = DefaultColHeight;
	//}}AFX_DATA_INIT
}


/////////////////////////////////////////////////////////////////////////////
// CSDIGridPropPage::DoDataExchange - Moves data between page and properties

void CSDIGridPropPage::DoDataExchange(CDataExchange* pDX)
{
	//{{AFX_DATA_MAP(CSDIGridPropPage)
	DDP_Text(pDX, IDC_NUM_ROWS, m_numRows, _T("NumRows") );
	DDX_Text(pDX, IDC_NUM_ROWS, m_numRows);
	DDP_Text(pDX, IDC_NUM_COLS, m_numCols, _T("NumCols") );
	DDX_Text(pDX, IDC_NUM_COLS, m_numCols);
	DDP_Check(pDX, IDC_COL0_FIXED_CHECK, m_ColZeroIsFixed, _T("ColZeroIsFixed") );
	DDX_Check(pDX, IDC_COL0_FIXED_CHECK, m_ColZeroIsFixed);
	DDP_Text(pDX, IDC_ADDRESS_HIDDEN, m_Address, _T("GetAddress") );
	DDX_Text(pDX, IDC_ADDRESS_HIDDEN, m_Address);
	DDP_Text(pDX, IDC_COL_HEIGHT, m_FixedColHeight, _T("FixedColHeight") );
	DDX_Text(pDX, IDC_COL_HEIGHT, m_FixedColHeight);
	//}}AFX_DATA_MAP
	DDP_PostProcessing(pDX);
}


/////////////////////////////////////////////////////////////////////////////
// CSDIGridPropPage message handlers

void CSDIGridPropPage::OnSetColumnButton() 
{
    //Get pointer back to the control;
    CSDIGridCtrl* pControl = (CSDIGridCtrl*)m_Address;

    CSetColumn SetColumn;
    SetColumn.m_NumCols = pControl->GetNumCols();

    //Get existing grid headings;
    CObList* pItems = pControl->GetColHeadings();
    SetColumn.SetHeadings(pItems);

    //Pass headings onto SetColumn;
    if (SetColumn.DoModal() == IDOK)
    {
        pControl->SetColHeadings();
    }
}

void CSDIGridPropPage::OnSetRowButton() 
{
    //Get pointer back to the control;
    CSDIGridCtrl* pControl = (CSDIGridCtrl*)m_Address;

    CSetRowHeading SetRowHeading;
    SetRowHeading.m_NumRows = pControl->GetNumRows();

    //Get existing grid headings;
    CObList* pItems = pControl->GetRowHeadings();
    SetRowHeading.SetHeadings(pItems);

    //Pass headings onto SetRowHeading;
    if (SetRowHeading.DoModal() == IDOK)
    {
        pControl->SetRowHeadings();
    }
}

void CSDIGridPropPage::RealignColsOrRows(CObList* pItems, int MaxItems)
{
    int NumCurrentItems = pItems->GetCount();

    //Are we adding or deleting items??
    if (NumCurrentItems < MaxItems)
    {
        //We need to add items;
        int NumShort = MaxItems - NumCurrentItems;

        for (int Index = 0; Index < NumShort; Index++)
        {
            CellDataType* pNewItem = new CellDataType;

            //Initiaize new structure;
            pNewItem->Id = pItems->GetCount();
            pNewItem->ColWidth = DefaultColWidth;
            pNewItem->ItemType = EditData;
            pItems->AddTail((CObject *)pNewItem);
        }
    }
    else //We are deleting items;
    {
        int NumToRemove = NumCurrentItems - MaxItems;

        for (int Index = 0; Index < NumToRemove; Index++)
        {
            if (pItems->GetCount() > 0)
            {
		        //Get last element;
		        CellDataType *pExtraObject = (CellDataType *)pItems->GetTail();

		        //Delete the object pointer;
		        pItems->RemoveTail();

    		    //Delete the object;
	    	    delete pExtraObject;
            }
        }
    }
}

void CSDIGridPropPage::OnChangeNumCols() 
{
    //Get pointer back to the control;
    CSDIGridCtrl* pControl = (CSDIGridCtrl*)m_Address;

    CObList* pItems = pControl->GetColHeadings();
    RealignColsOrRows(pItems, m_numCols);
}

void CSDIGridPropPage::OnChangeNumRows() 
{
    //Get pointer back to the control;
    CSDIGridCtrl* pControl = (CSDIGridCtrl*)m_Address;

    CObList* pItems = pControl->GetRowHeadings();
    RealignColsOrRows(pItems, m_numRows);
}
